@@ -15,6 +15,7 @@ HEADER_SIZE = 24
 STRUCT_HEADER = '!BBHBBHLLQ'
 STRUCT_GET = STRUCT_HEADER + '%ds'
 STRUCT_SET = STRUCT_HEADER + 'LL%ds%ds'
+STRUCT_FLUSH = STRUCT_HEADER + 'L'
 
 REQUEST_MAGIC = 0x80
 RESPONSE_MAGIC = 0x81
@@ -26,6 +27,24 @@ COMMAND_GETK = 0x0C
 COMMAND_GETKQ = 0x0D
 COMMAND_SET = 0x01
 COMMAND_SETQ = 0x11
+COMMAND_FLUSH = 0x08
+COMMAND_FLUSHQ = 0x18
+
+
+class MemcachedKeyError(Exception):
+    pass
+
+
+class MemcachedKeyLengthError(MemcachedKeyError):
+    pass
+
+
+class MemcachedKeyTypeError(MemcachedKeyError):
+    pass
+
+
+class MemcachedValueLengthError(Exception):
+    pass
 
 
 class MemcacheConnection(object):
@@ -48,12 +67,18 @@ class MemcacheConnection(object):
         established until you specifically call :func:`open() <gmemcache.MemcacheConnection.open>`.
     :param bool serializer: (optional) The class used to serialize the value to be cached.
         :class:`MsgpackSerializer <gmemcache.MsgpackSerializer>` is used by default.
+    :param int max_key_length: (optional) The maximum length of the cache key.
+    :param int max_value_length: (optional) The maximum length of the cache value.
     """
 
-    def __init__(self, hosts, timeout=5, lazy=False, serializer=MsgpackSerializer):
+    def __init__(self, hosts, timeout=5, lazy=False,
+                 serializer=MsgpackSerializer,
+                 max_key_length=250, max_value_length=1000**2):
         self._hosts = hosts
         self._timeout = timeout
         self._serializer = serializer()
+        self._max_key_length = max_key_length
+        self._max_value_length = max_value_length
 
         self._sockets = None
         self._ring = HashRing(self._hosts)
@@ -122,6 +147,8 @@ class MemcacheConnection(object):
 
         assert self._sockets is not None, 'The connection has not been created'
 
+        self._validate_key(key)
+
         host = self._ring.get_node(key)
         sock = self._sockets[host]
         try:
@@ -148,6 +175,7 @@ class MemcacheConnection(object):
         """Retreives multiple records with the specified keys.
 
         :param list keys: Cache keys.
+        :rtype: dict
         """
 
         assert self._sockets is not None, 'The connection has not been created'
@@ -157,6 +185,7 @@ class MemcacheConnection(object):
 
         divided_keys = defaultdict(list)
         for key in keys:
+            self._validate_key(key)
             divided_keys[self._ring.get_node(key)].append(key)
 
         msgs = defaultdict(str)
@@ -214,13 +243,18 @@ class MemcacheConnection(object):
         :param str key: Cache key.
         :param value: Value to be cached.
         :param int lifetime: The number of seconds until the records will expire.
+        :rtype: bool
         """
 
         assert self._sockets is not None, 'The connection has not been created'
 
+        self._validate_key(key)
+
         host = self._ring.get_node(key)
         sock = self._sockets[host]
+
         packed_value = self._serialize(value)
+        self._validate_value(packed_value)
 
         try:
             sock.sendall(struct.pack(STRUCT_SET % (len(key), len(packed_value)),
@@ -246,6 +280,7 @@ class MemcacheConnection(object):
 
         :param dict data: Records to be cached.
         :param int lifetime: The number of seconds until the records will expire.
+        :rtype: bool
         """
 
         assert self._sockets is not None, 'The connection has not been created'
@@ -255,13 +290,15 @@ class MemcacheConnection(object):
 
         divided_data = defaultdict(dict)
         for (key, value) in data.iteritems():
+            self._validate_key(key)
             divided_data[self._ring.get_node(key)][key] = value
 
         msgs = defaultdict(str)
         for (host, host_data) in divided_data.iteritems():
             for (n, (key, value)) in enumerate(host_data.iteritems()):
-                host = self._ring.get_node(key)
                 packed_value = self._serialize(value)
+                self._validate_value(packed_value)
+
                 if n != len(host_data) - 1:
                     msgs[host] += struct.pack(STRUCT_SET % (len(key), len(packed_value)),
                                               REQUEST_MAGIC,
@@ -308,6 +345,34 @@ class MemcacheConnection(object):
 
         return retval
 
+    def flush_all(self):
+        """Disable all stored items.
+
+        :rtype: bool
+        """
+
+        assert self._sockets is not None, 'The connection has not been created'
+
+        for host in self._hosts:
+            sock = self._sockets[host]
+            try:
+                sock.sendall(struct.pack(STRUCT_FLUSH,
+                                         REQUEST_MAGIC,
+                                         COMMAND_FLUSH,
+                                         0, 0, 0, 0, 0, 0, 0, 0))
+
+
+                resp = self._get_response(sock)
+
+            except:
+                self.reconnect(host)
+                raise
+
+            if resp['status'] == STATUS_SUCCESS:
+                return True
+            else:
+                return False
+
     def _serialize(self, value):
         if self._serializer:
             return self._serializer.serialize(value)
@@ -346,6 +411,20 @@ class MemcacheConnection(object):
 
         return value
 
+    def _validate_key(self, key):
+        if not isinstance(key, str):
+            raise MemcachedKeyTypeError('Key must be str()')
+
+        if len(key) > self._max_key_length:
+            raise MemcachedKeyLengthError(
+                'Key length must be less than %d' % self._max_key_length)
+
+    def _validate_value(self, val):
+        if len(val) > self._max_value_length:
+            raise MemcachedValueLengthError(
+                'Value length must be less than %d' % self._max_value_length
+            )
+
 
 class MemcachePoolConnection(MemcacheConnection, gsocketpool.Connection):
     """Memcache connection wrapper class for `gsocketpool <https://github.com/studio-ousia/gsocketpool>`_.
@@ -365,7 +444,10 @@ class MemcachePoolConnection(MemcacheConnection, gsocketpool.Connection):
     :param int timeout: (optional) Timeout.
     :param bool serializer: (optional) The class used to serialize the value to be cached.
         :class:`MsgpackSerializer <gmemcache.MsgpackSerializer>` is used by default.
+    :param int max_key_length: (optional) The maximum length of the cache key.
+    :param int max_value_length: (optional) The maximum length of the cache value.
     """
 
-    def __init__(self, hosts, timeout=5, serializer=MsgpackSerializer):
+    def __init__(self, hosts, timeout=5, serializer=MsgpackSerializer,
+                 max_key_length=250, max_value_length=1000**2):
         MemcacheConnection.__init__(self, hosts, timeout, lazy=False, serializer=serializer)
